'use strict';

exports.__esModule = true;

var _path = require('path');

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _pify = require('pify');

var _pify2 = _interopRequireDefault(_pify);

var _isDir = require('../lib/is-dir');

var _isDir2 = _interopRequireDefault(_isDir);

var _getPkg = require('../lib/get-pkg');

var _getPkg2 = _interopRequireDefault(_getPkg);

var _error = require('../lib/error');

var _error2 = _interopRequireDefault(_error);

var _logStats = require('../lib/webpack/log-stats');

var _runWebpack = require('../lib/webpack/run-webpack');

var _runWebpack2 = _interopRequireDefault(_runWebpack);

var _asyncCommand = require('../lib/async-command');

var _asyncCommand2 = _interopRequireDefault(_asyncCommand);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const pRimRaf = (0, _pify2.default)(_rimraf2.default);

exports.default = (0, _asyncCommand2.default)({
  command: 'build [src] [dest]',

  desc: 'Create a production build in build/',

  builder: {
    cwd: {
      description: 'A directory to use instead of $PWD.',
      default: '.'
    },
    src: {
      description: 'Entry file (index.js)',
      default: 'src'
    },
    dest: {
      description: 'Directory root for output',
      default: 'dist'
    },
    workers: {
      description: 'Add a Service Workers to the application.',
      alias: 'w',
      type: 'boolean',
      default: true
    },
    clean: {
      description: 'Clear output directory before building.',
      type: 'boolean',
      default: true
    },
    json: {
      description: 'Generate build statistics for analysis.',
      type: 'boolean',
      default: false
    }
  },

  handler(argv) {
    return _asyncToGenerator(function* () {
      const spinner = (0, _ora2.default)({
        text: 'Preparing build things...',
        color: 'magenta'
      }).start();

      const cwd = (0, _path.resolve)(argv.cwd);
      const modules = (0, _path.resolve)(cwd, 'node_modules');

      if (!(0, _isDir2.default)(modules)) {
        (0, _error2.default)('No `node_modules` found! Please run `npm install` before continuing.', spinner);
        return process.exit(1);
      }

      if (argv.clean) {
        const dest = (0, _path.resolve)(cwd, argv.dest || 'dist');
        yield pRimRaf(dest);
      }

      const pkg = yield (0, _getPkg2.default)(argv.cwd);
      const newArgv = Object.assign({}, argv, { production: true, pkg });

      spinner.color = 'yellow';
      spinner.text = 'Running compiler...';

      try {
        const results = yield (0, _runWebpack2.default)(newArgv);
        (0, _logStats.endBuildMessage)(results, spinner);

        (0, _logStats.showStats)(results);
        (0, _logStats.endMessage)(results.toJson().assets);
      } catch (err) {
        console.error('\n' + err);
      }
    })();
  }
});