'use strict';

exports.__esModule = true;

var _path = require('path');

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _gittar = require('gittar');

var _gittar2 = _interopRequireDefault(_gittar);

var _inquirer = require('inquirer');

var _validateNpmPackageName = require('validate-npm-package-name');

var _validateNpmPackageName2 = _interopRequireDefault(_validateNpmPackageName);

var _gitUserLocal = require('git-user-local');

var _gitUserLocal2 = _interopRequireDefault(_gitUserLocal);

var _replaceOwner = require('../lib/replace-owner');

var _replaceOwner2 = _interopRequireDefault(_replaceOwner);

var _isMissing = require('../lib/is-missing');

var _isMissing2 = _interopRequireDefault(_isMissing);

var _asyncCommand = require('../lib/async-command');

var _asyncCommand2 = _interopRequireDefault(_asyncCommand);

var _which = require('../lib/which');

var _which2 = _interopRequireDefault(_which);

var _error = require('../lib/error');

var _error2 = _interopRequireDefault(_error);

var _isDir = require('../lib/is-dir');

var _isDir2 = _interopRequireDefault(_isDir);

var _extractTar = require('../lib/extract-tar');

var _extractTar2 = _interopRequireDefault(_extractTar);

var _install = require('../lib/install');

var _install2 = _interopRequireDefault(_install);

var _getPkg = require('../lib/get-pkg');

var _getPkg2 = _interopRequireDefault(_getPkg);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const ORG = 'PolymerX-skeleton-templates';

const getAuthor = (() => {
  var _ref = _asyncToGenerator(function* () {
    var _ref2 = yield (0, _gitUserLocal2.default)();

    const user = _ref2.user;

    return `${user ? user.name : ''} <${user ? user.email : ''}>`;
  });

  return function getAuthor() {
    return _ref.apply(this, arguments);
  };
})();

const capitalize = str => str.charAt(0).toUpperCase() + str.substring(1);
const completeData = (() => {
  var _ref3 = _asyncToGenerator(function* (argv, spinner) {
    const questions = (0, _isMissing2.default)(argv);
    if (questions.length > 0) {
      spinner.warn('Missing commang arguments. Prompting...');
      spinner.info('Alternatively, run `polymerx create --help` for usage info.');
    }
    const response = yield (0, _inquirer.prompt)(questions);
    return Object.assign({}, response);
  });

  return function completeData(_x, _x2) {
    return _ref3.apply(this, arguments);
  };
})();

const checkExistOrForce = (() => {
  var _ref4 = _asyncToGenerator(function* (exists, isForceEnabled, spinner) {
    if (exists && !isForceEnabled) {
      (0, _error2.default)(`Refusing to overwrite current directory!
Please specify a different destination or use the \`--force\` flag`, spinner);
      return process.exit(1);
    }

    if (exists && isForceEnabled) {
      var _ref5 = yield (0, _inquirer.prompt)({
        type: 'confirm',
        name: 'enableForce',
        message: `You are using '--force'. Do you wish to continue?`,
        default: false
      });

      const enableForce = _ref5.enableForce;


      if (!enableForce) {
        (0, _error2.default)('Refusing to overwrite current directory!', spinner);
        return process.exit(1);
      }

      spinner.info('Initializing project in the current directory!');
    }
  });

  return function checkExistOrForce(_x3, _x4, _x5) {
    return _ref4.apply(this, arguments);
  };
})();

const getStarted = (destFolder, isYarn) => {
  const prefix = isYarn ? 'yarn' : 'npm run';
  const text = `
    To get started, cd into the new directory:
      cd ${destFolder}
    To start a the dev server with live-reload (for browsers WITHOUT ES6 module support):
      ${prefix} dev
    To create a production build (in ./dist):
      ${prefix} build
    To start a production HTTP server:
      ${prefix} start
  `;
  return console.log(text.trim().replace(/^\t+/gm, ''), '\n');
};

exports.default = (0, _asyncCommand2.default)({
  command: 'create [template] [dest]',

  desc: 'Create a new application.',

  builder: {
    cwd: {
      description: 'A directory to use instead of $PWD.',
      default: '.'
    },
    name: {
      description: 'The application\'s name'
    },
    author: {
      description: 'Author of the app',
      default: null
    },
    force: {
      description: 'Force option to create the directory for the new app',
      type: 'boolean',
      default: false
    },
    yarn: {
      description: 'Use \'yarn\' instead of \'npm\'',
      type: 'boolean',
      default: false
    },
    git: {
      description: 'Initialize version control using git',
      type: 'boolean',
      default: false
    },
    install: {
      description: 'Install dependencies',
      type: 'boolean',
      default: true
    }
  },

  handler(argv) {
    return _asyncToGenerator(function* () {
      const spinner = (0, _ora2.default)({
        text: 'Analyzing args...',
        color: 'magenta'
      }).start();

      const restData = yield completeData(argv, spinner);
      Object.assign(argv, restData);

      const cwd = (0, _path.resolve)(argv.cwd);
      const packageName = argv.name || argv.dest;
      const destFolder = argv.dest || (0, _path.dirname)(cwd);
      const target = (0, _path.resolve)(cwd, destFolder);
      const author = argv.author || (yield getAuthor());
      const installDeps = argv.install;

      const exists = (0, _isDir2.default)(target);
      const isForceEnabled = argv.force;

      yield checkExistOrForce(exists, isForceEnabled, spinner);

      const repo = argv.template.includes('/') ? argv.template : `${ORG}/${argv.template}`;

      spinner.info(`Getting ${repo}...`);

      var _validateName = (0, _validateNpmPackageName2.default)(packageName);

      const errors = _validateName.errors;

      if (errors) {
        errors.unshift(`Invalid package name: ${packageName}`);
        (0, _error2.default)(errors.map(capitalize).join('\n  ~ '), spinner);
        return process.exit(1);
      }

      const archive = yield _gittar2.default.fetch(repo).catch(function (err) {
        const finalErr = err || { message: 'An error occured while fetching template.' };
        (0, _error2.default)(finalErr.code === 404 ? `Could not find repository: ${repo}` : finalErr.message, spinner);
        return process.exit(1);
      });

      spinner.text = '⚡️  Creating project';
      spinner.color = 'yellow';
      spinner.start();

      const keeps = yield (0, _extractTar2.default)(archive, target);
      const isRepoEmpty = keeps.length <= 0;
      if (isRepoEmpty) {
        (0, _error2.default)(`No \`template\` directory found within ${repo}!`, spinner);
        return process.exit();
      }

      spinner.color = 'magenta';
      spinner.text = '⚙️  Making changes to files...';
      yield (0, _replaceOwner2.default)(keeps, packageName, author);

      const pkg = yield (0, _getPkg2.default)(target);
      if (!pkg) {
        spinner.warn('⚠️ Could not locate `package.json` file!');
      }

      const isYarn = argv.yarn && (yield (0, _which2.default)('yarn').catch(function () {
        (0, _error2.default)('Yarn not found. Please remove the "--yarn" flag or install yarn.', spinner);
        process.exit(1);
      }));

      if (installDeps) {
        spinner.color = 'cyan';
        spinner.text = '📦  Installing dependencies...';
        yield (0, _install2.default)(target, isYarn);
      }

      spinner.succeed('Everything cloned and installed correctly!\n');
      getStarted(destFolder, isYarn);
    })();
  }
});