'use strict';

exports.__esModule = true;

var _path = require('path');

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _getPort = require('get-port');

var _getPort2 = _interopRequireDefault(_getPort);

var _consoleClear = require('console-clear');

var _consoleClear2 = _interopRequireDefault(_consoleClear);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackDevServer = require('webpack-dev-server');

var _webpackDevServer2 = _interopRequireDefault(_webpackDevServer);

var _isDir = require('../is-dir');

var _isDir2 = _interopRequireDefault(_isDir);

var _logStats = require('./log-stats');

var _webpackBase = require('./webpack-base.config');

var _webpackBase2 = _interopRequireDefault(_webpackBase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const runProdCompiler = compiler => {
  return new Promise((resolve, reject) => compiler.run((() => {
    var _ref = _asyncToGenerator(function* (err, stats) {
      (0, _logStats.showStats)(stats);

      if (err || stats && stats.hasErrors()) {
        reject(_chalk2.default.red(`⚠️ build failed! ${err || ''}`));
      }

      yield new Promise(function (resolve) {
        return setTimeout(resolve, 20);
      });
      resolve(stats);
    });

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  })()));
};

const devBuild = (() => {
  var _ref2 = _asyncToGenerator(function* (argv) {
    const config = (0, _webpackBase2.default)(argv);
    const userPort = Number(process.env.PORT || config.devServer.port) || 8080;
    const port = yield (0, _getPort2.default)(userPort);
    const compiler = (0, _webpack2.default)(config);

    return new Promise(function (resolve, reject) {
      compiler.plugin('done', function (stats) {
        const devServer = config.devServer;

        const protocol = process.env.HTTPS || devServer.https ? 'https' : 'http';
        const userHost = process.env.HOST || devServer.host || 'localhost';
        const host = userHost === '0.0.0.0' ? 'localhost' : userHost;
        const serverAddr = `${protocol}://${host}:${_chalk2.default.bold(port)}`;


        (0, _consoleClear2.default)();
        (0, _logStats.endMessage)(stats.toJson().assets, stats.hasErrors());
        (0, _logStats.detailMessage)(stats.hasErrors(), {
          port,
          userPort,
          serverAddr,
          nomodule: argv.nomodule,
          assets: stats.toJson({ assets: true }).assets
        });
        (0, _logStats.showStats)(stats);
      });

      compiler.plugin('failed', reject);
      new _webpackDevServer2.default(compiler, config.devServer).listen(port);
    });
  });

  return function devBuild(_x3) {
    return _ref2.apply(this, arguments);
  };
})();

const prodBuild = (() => {
  var _ref3 = _asyncToGenerator(function* (argv) {
    const config = (0, _webpackBase2.default)(argv);
    const compiler = (0, _webpack2.default)(config);

    return runProdCompiler(compiler);
  });

  return function prodBuild(_x4) {
    return _ref3.apply(this, arguments);
  };
})();

exports.default = argv => {
  const cwd = (0, _path.resolve)(argv.cwd || process.cwd());

  const src = (0, _path.resolve)(cwd, argv.src);
  const srcDir = (0, _isDir2.default)(src) ? src : (0, _path.dirname)(src);

  const newArgv = Object.assign({}, argv, {
    isProd: argv.production,
    cwd,
    src,
    srcDir,
    nomodule: argv.nomodule
  });

  const fn = argv.production ? prodBuild : devBuild;
  return fn(newArgv);
};