'use strict';

exports.__esModule = true;

var _path = require('path');

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackMerge = require('webpack-merge');

var _webpackMerge2 = _interopRequireDefault(_webpackMerge);

var _workboxWebpackPlugin = require('workbox-webpack-plugin');

var _copyWebpackPlugin = require('copy-webpack-plugin');

var _copyWebpackPlugin2 = _interopRequireDefault(_copyWebpackPlugin);

var _htmlWebpackPlugin = require('html-webpack-plugin');

var _htmlWebpackPlugin2 = _interopRequireDefault(_htmlWebpackPlugin);

var _htmlWebpackExcludeAssetsPlugin = require('html-webpack-exclude-assets-plugin');

var _htmlWebpackExcludeAssetsPlugin2 = _interopRequireDefault(_htmlWebpackExcludeAssetsPlugin);

var _scriptExtHtmlWebpackPlugin = require('script-ext-html-webpack-plugin');

var _scriptExtHtmlWebpackPlugin2 = _interopRequireDefault(_scriptExtHtmlWebpackPlugin);

var _cleanWebpackPlugin = require('clean-webpack-plugin');

var _cleanWebpackPlugin2 = _interopRequireDefault(_cleanWebpackPlugin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const renderHtmlPlugins = (outputPath, isProd, src) => [new _htmlWebpackPlugin2.default({
  filename: (0, _path.resolve)(outputPath, 'index.html'),
  template: `!!ejs-loader!${(0, _path.resolve)(src, 'index.html')}`,
  minify: isProd && {
    collapseWhitespace: true,
    removeScriptTypeAttributes: true,
    removeRedundantAttributes: true,
    removeStyleLinkTypeAttributes: true,
    removeComments: true
  },
  inject: true,
  compile: true,
  excludeAssets: [/(bundle|polyfills)(\..*)?\.js$/],
  paths: {
    webcomponents: './vendor/webcomponents-loader.js'
  }
}), new _htmlWebpackExcludeAssetsPlugin2.default(), new _scriptExtHtmlWebpackPlugin2.default({
  defaultAttribute: 'defer'
})];

const shared = argv => {
  const isProd = argv.isProd,
        src = argv.src,
        srcDir = argv.srcDir,
        dest = argv.dest,
        pkg = argv.pkg,
        cwd = argv.cwd,
        https = argv.https,
        workers = argv.workers;


  const ENV = isProd ? 'production' : 'development';
  const OUTPUT_PATH = isProd ? (0, _path.resolve)(dest || 'dist') : (0, _path.resolve)(src || 'src');

  const HOST = process.env.HOST || argv.host;
  const PORT = process.env.PORT || argv.port;
  const FULL_HOST = `http://${HOST}:${PORT}`;
  const ENTRY = isProd ? [(0, _path.resolve)(src || 'src')] : [OUTPUT_PATH].concat([`webpack-dev-server/client?${FULL_HOST}`, 'webpack/hot/dev-server']);

  const processEnv = {
    NODE_ENV: JSON.stringify(ENV),
    appVersion: JSON.stringify(pkg.version)
  };

  const copyStatics = {
    copyWebcomponents: [{
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/webcomponents-loader.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor'),
      flatten: true
    }, {
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/webcomponents-bundle.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor'),
      flatten: true
    }, {
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/bundles/webcomponents-ce.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor', 'bundles'),
      flatten: true
    }, {
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/bundles/webcomponents-sd-ce.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor', 'bundles'),
      flatten: true
    }, {
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/bundles/webcomponents-sd-ce-pf.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor', 'bundles'),
      flatten: true
    }, {
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/bundles/webcomponents-sd.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor', 'bundles'),
      flatten: true
    }],
    copyOthers: [{
      from: 'assets/**',
      context: (0, _path.resolve)(srcDir),
      to: OUTPUT_PATH
    }, {
      from: (0, _path.resolve)(srcDir, 'index.html'),
      to: OUTPUT_PATH,
      flatten: true
    }, {
      from: (0, _path.resolve)(srcDir, 'manifest.json'),
      to: OUTPUT_PATH,
      flatten: true
    }]
  };

  const sharedPlugins = [new _webpack2.default.DefinePlugin({ 'process.env': processEnv }), ...renderHtmlPlugins(OUTPUT_PATH, isProd, src)];

  const devPlugins = [new _copyWebpackPlugin2.default(copyStatics.copyWebcomponents), new _webpack2.default.HotModuleReplacementPlugin(), new _webpack2.default.NamedModulesPlugin()];

  const buildPlugins = [new _cleanWebpackPlugin2.default([(0, _path.resolve)(OUTPUT_PATH)], {
    allowExternal: true, verbose: false
  }), new _copyWebpackPlugin2.default([].concat(copyStatics.copyWebcomponents, copyStatics.copyOthers))].concat(workers ? new _workboxWebpackPlugin.GenerateSW({
    swDest: (0, _path.join)(OUTPUT_PATH, 'sw.js'),
    skipWaiting: true
  }) : []);

  const plugins = sharedPlugins.concat(isProd ? buildPlugins : devPlugins);

  return {
    mode: ENV,
    entry: ENTRY,
    output: {
      path: OUTPUT_PATH,
      filename: 'bundle.js'
    },
    devtool: 'cheap-source-map',
    resolve: {
      modules: ['node_modules', (0, _path.resolve)(__dirname, '../../../node_modules')]
    },
    resolveLoader: {
      modules: [(0, _path.resolve)(__dirname, '../../../node_modules'), (0, _path.resolve)(cwd, 'node_modules')]
    },
    module: {
      rules: [{
        test: /\.html$/,
        use: ['raw-loader']
      }, {
        test: /\.pcss$/,
        use: ['raw-loader', 'postcss-loader']
      }, {
        test: /\.js$/,
        exclude: /(node_modules)/,
        use: {
          loader: 'babel-loader',
          options: {
            presets: [[require.resolve('babel-preset-env'), {
              targets: {
                browsers: ['>=1%', 'not ie 11', 'not op_mini all']
              },
              debug: !isProd
            }]],
            plugins: [[require.resolve('babel-plugin-transform-object-rest-spread'), { useBuiltIns: true }]]
          }
        }
      }]
    },
    plugins,
    devServer: {
      hot: true,
      compress: true,
      contentBase: OUTPUT_PATH,
      overlay: {
        errors: true
      },
      stats: 'minimal',
      port: PORT,
      host: HOST,
      https,
      disableHostCheck: true,
      historyApiFallback: true,
      quiet: true,
      clientLogLevel: 'none',
      watchOptions: {
        ignored: [(0, _path.resolve)(cwd, 'dist'), (0, _path.resolve)(cwd, 'node_modules')]
      }
    }
  };
};

exports.default = (argv = {}) => (0, _webpackMerge2.default)(shared(argv));